import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.Serializable;
import java.beans.*;


 /**
 *  <h4>DoubleParameterBean.java  - A bean to set double values using a textfield or a scrollbar.</h4>
 *  <br/>
 *  Copyright (C) 2011 Guido Kramann<br/>
 *  kramann@fh-brandenburg.de<br/>
 *  http://www.kramann.info<br/>
 *  <br/>
 *  <p>  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  </p>
 *  <p>
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  </p>
 *  <p>
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  </p>
 */
public class DoubleParameterBean extends BasisBean
{
    private JLabel   aufkleber;
    private Textfeld textfeld;
    private Schieber schieber;

    /**
    Fr die einzustellenden Parameter m&uuml;ssen get- und set-Methoden bereitgestellt werden.
    */
    private double wert;
    private double minwert;
    private double maxwert;
    private String bezeichnung = null;

    public void aktualisieren()
    {
        textfeld.setText(""+wert);
        schieber.setWert(wert,minwert,maxwert);
    }

    public void setWert(double wert) 
    {     
        this.wert = wert;
        aktualisieren();
    }
    public double getWert() 
    {
        return this.wert;
    }

    public void setMinwert(double minwert) 
    {
        this.minwert=minwert;
        aktualisieren();
    }
    public double getMinwert() 
    {
        return this.minwert;
    }

    public void setMaxwert(double maxwert) 
    {
        this.maxwert=maxwert;
        aktualisieren();
    }
    public double getMaxwert() 
    {
        return this.maxwert;
    }


    public void setBezeichnung(String bezeichnung) 
    {
        aufkleber.setText(bezeichnung);
        firePropertyChange("bezeichnung",this.bezeichnung,bezeichnung);
        this.bezeichnung=bezeichnung;
    }
    public String getBezeichnung() 
    {
        return this.bezeichnung;
    }

    /**
    <h4>Aufruf durch Textfeld, aktualisiert Schieber und schickt PropertyChange-Event.</h4>
    */
    private void setWertIntern(String text)
    {
        double x = this.wert;
        double wert_alt = this.wert;
        boolean ok = true;
        try
        {
            x = Double.parseDouble(text);
        }
        catch(Exception e)
        {
            ok = false;
        }
        if( ok==true )
            this.wert = x;

        firePropertyChange("wert",wert_alt,this.wert);
        schieber.setWert(wert,minwert,maxwert);
    }

    /**
    <h4>Aufruf durch Schieber, aktualisiert Textfeld und schickt PropertyChange-Event.</h4>
    */
    public void setWertIntern(int iwert,int max_iwert)
    {
        double wert_alt = this.wert;
        this.wert = minwert + ((maxwert-minwert)*(double)iwert) / (double)max_iwert;

        firePropertyChange("wert",wert_alt,this.wert);
        textfeld.setText(""+wert);
    }

    /**
    JavaBeans m&uuml;ssen einen leeren Kontruktor besitzen.
    */
    public DoubleParameterBean() 
    {
        int breite = 450;
        int hoehe  = 30;

        wert        = 0.0;
        bezeichnung = "Zahl";        

        minwert = 0.0;
        maxwert = 100.0;

        textfeld  = new Textfeld(""+wert);
        aufkleber = new JLabel(bezeichnung); 
        schieber  = new Schieber(300);


        setLayout(new GridLayout(1,3));

        setBounds(10,60,breite,hoehe);
        add(aufkleber);
        add(schieber);
        add(textfeld);

        setBackground( Color.BLUE );
        aufkleber.setForeground( Color.WHITE );
    } 

    /**<h4>Innere Klasse von DoubleParameterBean, repr&auml;sentiert Textfeld</h4>*/
    public class Textfeld extends TextField
    {
        Textfeld(String text)
        {
            super(text);
            enableEvents (AWTEvent.ACTION_EVENT_MASK);             
            enableEvents (AWTEvent.TEXT_EVENT_MASK);             
            enableEvents (AWTEvent.FOCUS_EVENT_MASK);             
        }

        protected void processEvent(AWTEvent e) 
        { 
             setWertIntern(getText());
        } 
    }

    /**<h4>Innere Klasse von DoubleParameterBean, repr&auml;sentiert Schieberegler</h4>*/
    public class Schieber extends JScrollBar
    {
        private int max_iwert;

        public void aktivieren()
        {
            addAdjustmentListener(new SchieberLauscher());
        }

        public void setWert(double wert, double minwert, double maxwert)
        {
            setValue( (int)((double)max_iwert*((wert-minwert)/(maxwert-minwert))) ); 
        }

        Schieber(int max_iwert)
        {
            super(JScrollBar.HORIZONTAL,0,10,0,max_iwert+10);        
            aktivieren();
            this.max_iwert = max_iwert;
        }

        /**Innere Klasse von Schieber, Listener fr Schieberegler-Bewegung*/
        public class SchieberLauscher implements AdjustmentListener 
        {        
            public void adjustmentValueChanged(AdjustmentEvent e) 
            {
                  setWertIntern(e.getValue(),max_iwert);
            }
        }
    }
}
